#include "wrap/thread.h"
#include "wrap/synch.h"
#include <stdlib.h>
#include <stdio.h>

struct info {
	int tid;
	const char *message;
};

struct lock *lock;
struct semaphore *done;

static struct info *make_info(int id, const char *message) {
	struct info *i = malloc(sizeof(*i));
	i->tid = id;
	i->message = message;
	return i;
}

static void delay(int n) {
	int x = 100000*n;
	while (x --> 0)
		n = x & n;
}

static void putline(const char *message) {
	/* lock_acquire(&lock[0]); */

	while (*message) {
		putchar(*message++);
		delay(1);
	}
	putchar('\n');
	delay(1);

	/* lock_release(&lock[0]); */
}

static void printer(void *input) {
	struct info *info = input;
	int i;

	for (i = 0; i < 100; i++) {
		/* lock_acquire(&lock[info->tid]); */
		/* lock_acquire(&lock[0]); */
		putline(info->message);
		/* lock_release(&lock[0]); */
		/* lock_release(&lock[info->tid]); */
	}

	sema_up(&done[info->tid]);
	free(info);
}


int main(void) {
	const char *m[] = {
		"This line is not written in gibberish",
		"We want every line to be perfectly readable",
		"The quick brown fox jumps over the lazy dog",
		"Lorem ipsum dolor sit amet",
		NULL,
	};
	const int N = sizeof(m)/sizeof(*m)-1;
	int i;

	lock = malloc(sizeof(*lock)*N);
	done = malloc(sizeof(*done)*N);

	for (i = 0; i < N; i++) {
		sema_init(&done[i], 0);
		lock_init(&lock[i]);
	}

	for (i = 0; i < N; i++) {
		thread_create("", 0, &printer, make_info(i, m[i]));
	}

	for (i = 0; i < N; i++) {
		sema_down(&done[i]);
	}

	for (i = 0; i < N; i++) {
		lock_destroy(&lock[i]);
		sema_destroy(&done[i]);
	}

	free(lock);
	free(done);

	return 0;
}
